from PIL import Image
import requests
import io
import base64

from .models import User


class AccountManager:
    def __init__(self, client):
        self.client = client

    def change_status(self, status: str) -> dict:
        """## Changes the users status.

        #### Args:
            status (str): The new status

        #### Returns:
            dict: The new status.
        """
        response = self.client._post("account/change_status", data={"status": status})
        return response

    def change_email(self, email: str) -> dict:
        """## Changes the users email.

        #### Args:
            email (str): The new email.

        #### Returns:
            dict: The new email.
        """
        response = self.client._post("/account/change_email", data={"email": email})
        return response

    def resend_validation_email(self, email: str) -> str:
        """## Resends a validation email.

        #### Args:
            email (str): The used email.

        #### Returns:
            str: The success status.
        """
        response = self.client._post(
            "/register/resend_validation_email", data={"email": email}
        )
        return response

    def change_password(self, new_password: str, old_password: str) -> dict:
        """## Changes a users password.

        #### Args:
            new_password (str): The new password.
            old_password (str): The old password.

        #### Returns:
            dict: The success status.
        """
        data = {"new_password": new_password, "old_password": old_password}
        response = self.client._post("/account/change_password", data=data)
        return response

    def settings(self) -> dict:
        """## Gets the users settings.

        #### Returns:
            dict: The settings
        """
        response = self.client._post("/account/settings", data={})
        return response["settings"]

    def active_devices(self) -> dict:
        """## Gets a users active devices.

        #### Returns:
            dict: The active devices.
        """
        response = self.client._post("/account/list_active_devices", data={})
        return response["devices"]

    def remove_device(self, device_id: str | int) -> dict:
        """## Deactivates an active device.

        #### Args:
            device_id (str | int): The device id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "account/deactivate_device", data={"device_to_remove": device_id}
        )
        return response

    def notifications(self, limit: int = 20, offset: int = 0) -> dict:
        """## Gets a users notifications.

        #### Args:
            limit (int, optional): The response limit. Defaults to 20.
            offset (int, optional): The response offset. Defaults to 0.

        #### Returns:
            dict: The notifications.
        """
        data = {"limit": limit, "offset": offset}
        response = self.client._post("notifications/get", data=data)
        return response["notifications"]

    def notification_count(self) -> int:
        """## Gets the notification count.

        #### Returns:
            int: The notification count.
        """
        response = self.client._post("notifications/count", data={})
        return int(response["count"])

    def location(self) -> dict:
        """## Gets the users location information.

        #### Returns:
            dict: The information.
        """
        response = self.client._post("/location/get", data={})
        return response["location"]

    def reset_profile_picture(self) -> dict:
        """## Resets the users profile picture.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post("account/reset_profile_image", data={})
        return response

    def change_profile_picture(self, url: str) -> User:
        """## Changes the users profile picture.

        #### Args:
            url (str): A image url.

        #### Returns:
            User: A user object.
        """
        response = requests.get(url)
        response.raise_for_status()

        with Image.open(io.BytesIO(response.content)) as image:

            min_dimension = min(image.width, image.height)
            scale_factor = 512 / min_dimension

            new_width = int(image.width * scale_factor)
            new_height = int(image.height * scale_factor)
            image = image.resize((new_width, new_height), Image.Resampling.LANCZOS)

            left, top = (new_width - 512) / 2, (new_height - 512) / 2
            right, bottom = left + 512, top + 512
            image = image.crop((left, top, right, bottom))

            buffered = io.BytesIO()
            image.save(buffered, format="PNG")

            image_base64 = base64.b64encode(buffered.getvalue()).decode("utf-8")

            response = self.client._post(
                "account/store_profile_image",
                data={"imgBase64": f"data:image/png;base64,{image_base64}"},
            )

            return User(self.client, response["user"])

    def statistics(self, company_id: str | int) -> dict:
        """## Gets company statistics.

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: The companies statistics
        """
        response = self.client._post("manage/accounts", data={"company_id": company_id})
        return response



from .models import Company


class CompanyManager:
    def __init__(self, client) -> None:
        self.client = client

    def info(self, company_id: str | int) -> Company:
        """## Gets the info of a company.

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            Company: A company object.
        """
        response = self.client._post("company/details", data={"company_id": company_id})
        return Company(self.client, response["company"])

    def member(self) -> list:
        """## Lists the companies of the logged in user.

        #### Returns:
            list: The company objects in a list.
        """
        response = self.client._post("company/member", data={"no_cache": True})
        return [Company(self.client, data) for data in response["companies"]]

    def get_settings(self, company_id: str | int) -> dict:
        """## Gets the settings of a company.

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: The companies settings.
        """
        response = self.client._post(
            "company/settings", data={"company_id": company_id}
        )
        return response["settings"]

    def email_templates(self, company_id: str | int) -> dict:
        """## Gets the email templates of the company.

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: Return a dict i think
        """
        response = self.client._post(
            "server/get_email_templates", data={"company_id": company_id}
        )
        return response["templates"]

    def get_ldaps(self, company_id: str | int) -> dict:
        """## Gets the companies ldaps [untested].

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: [untested]
        """
        response = self.client._post(
            "connections/servers", data={"company_id": company_id}
        )
        return response["servers"]

    def delete(self, company_id: str | int) -> dict:
        """## Deletes the company [dangerous!].

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "server/delete_company", data={"company_id": company_id}
        )
        return response

    def quit(self, company_id: str | int) -> dict:
        """## Leaves a company [dangerous!].

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post("company/quit", data={"company_id": company_id})
        return response

    def list_features(self, company_id: str | int) -> dict:
        """## Lists company features.

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: Company features
        """
        response = self.client._post(
            "server/list_company_features", data={"company_id": company_id}
        )
        return response["company_features"]

    def get_market(self, company_id: str | int) -> dict:
        """## Gets the companies market.

        #### Args:
            company_id (str | int): The companies id.

        #### Returns:
            dict: The market
        """
        response = self.client._post(
            "manage/get_company_market", data={"company_id": company_id}
        )
        return response["market"]



"""
StashConnect
~~~~~~~~~~~~
An API wrapper for [Stashcat](https://stashcat.com/en).

:copyright: (c) 2024-present BuStudios
:license: MIT, see LICENSE.
"""

__version__ = "0.9.7.6"

from .client import *



class AuthManager:
    def __init__(self, client):
        self.client = client

    def _login(self, email: str, password: str, app_name: str) -> dict:
        """## Logs into a account.

        #### Args:
            email (str): The users email.
            password (str): The users password.
            app_name (str): The app name to be used.

        #### Returns:
            dict: User data.
        """
        data = {
            "email": email,
            "password": password,
            "app_name": app_name,
            "encrypted": "true",
            "callable": "true",
        }

        response = self.client._post("auth/login", data=data, auth=False)
        return response

    def verify_login(
        self, app_name: str = None, encrypted: bool = True, callable: bool = True
    ) -> dict:
        """## Verifies if the user has logged in successfully.

        #### Args:
            app_name (str, optional): The devices app name. Defaults to None.
            encrypted (bool, optional): The devices encryption status. Defaults to True.
            callable (bool, optional): The devices callable status. Defaults to True.

        #### Returns:
            dict: The input args.
        """
        data = {
            "app_name": self.client.app_name if app_name is None else app_name,
            "encrypted": encrypted,
            "callable": callable,
        }

        return data



import Crypto.Cipher
import Crypto.Cipher.AES
import Crypto.Cipher.PKCS1_OAEP
import Crypto.Protocol
import Crypto.PublicKey
import Crypto.PublicKey.RSA
import Crypto

import Crypto.Random
import Crypto.Util
import Crypto.Util.Padding

import base64


class CryptoUtils:

    def encrypt_aes(plain: bytes, key: bytes, iv: bytes) -> bytes:
        """## Encrypts the provided plaintext using AES.

        #### Args:
            plain (bytes): Plaintext data to be encrypted.
            key (bytes): The key used for AES encryption
            iv (bytes): The iv used for AES encryption. (16 bytes)

        #### Returns:
            bytes: The encrypted data as bytes.
        """
        padded = Crypto.Util.Padding.pad(plain, Crypto.Cipher.AES.block_size)
        encryptor = Crypto.Cipher.AES.new(key, Crypto.Cipher.AES.MODE_CBC, iv=iv)
        return encryptor.encrypt(padded)

    def decrypt_aes(encrypted: bytes, key: bytes, iv: bytes) -> bytes:
        """## Decrypts the provided data using AES.

        #### Args:
            encrypted (bytes): The encrypted data to be decrypted.
            key (bytes): The key used for AES decryption.
            iv (bytes): The iv used for AES decryption. (16 bytes)

        #### Returns:
            bytes: The decoded plaintext data.
        """
        decryptor = Crypto.Cipher.AES.new(key, Crypto.Cipher.AES.MODE_CBC, iv=iv)
        decrypted = decryptor.decrypt(encrypted)
        return Crypto.Util.Padding.unpad(decrypted, Crypto.Cipher.AES.block_size)

    def decrypt_key(encrypted_key: bytes, private_key: bytes) -> bytes:
        """## Decrypts an RSA-encrypted key.

        #### Args:
            encrypted_key (bytes): The encrypted key data as bytes.
            private_key (bytes): The RSA private key object used for decryption

        #### Returns:
            bytes: The decrypted key as plaintext data.
        """
        decryptor = Crypto.Cipher.PKCS1_OAEP.new(private_key)
        return decryptor.decrypt(base64.b64decode(encrypted_key))

    def load_private_key(encrypted_key: bytes, encryption_password: str):
        """## Imports an RSA private key using a passphrase.

        #### Args:
            encrypted_key (bytes): The encrypted RSA private key.
            encryption_password (str): The passphrase used to decrypt the private key.

        #### Returns:
            The decrypted RSA private key object.
        """
        private_key = Crypto.PublicKey.RSA.import_key(
            encrypted_key, passphrase=encryption_password
        )
        return private_key



import Crypto.Cipher
import Crypto.Cipher.AES
import Crypto.Cipher.PKCS1_OAEP
import Crypto.Protocol
import Crypto.PublicKey
import Crypto.PublicKey.RSA
import Crypto

import Crypto.Random
import Crypto.Util
import Crypto.Util.Padding

import json
from typing import Generator

from .crypto_utils import CryptoUtils
from .models import Message


class MessageManager:
    def __init__(self, client):
        self.client = client

    def send(
        self,
        target: str | int,
        text: str,
        *,
        markdown: bool = True,
        files: str | int | list = None,
        urls: str | list = "",
        location: bool | tuple | list = None,
        encrypted: bool = True,
        **kwargs,
    ) -> Message:
        """## Sends a message.

        #### Args:
            target (str | int): The messages target location.
            text (str): The text to send.
            markdown (bool): Add markdown support. Defaults to True.
            files (str | int | list, optional): Files to send. Defaults to None.
            urls (str | list, optional): Url's to append to the message. Defaults to "".
            location (bool | tuple | list, optional): The location of the message. Defaults to None.
            encrypted (bool, optional): If the message should be encrypted. Defaults to True.

        #### Info:
            :The location needs to be set to (lat, lng) in a tuple or None.

        #### Returns:
            Message: A message object.
        """
        target_type = self.client.tools.get_type(target)

        if encrypted:
            if self.client._private_key is None:
                print(
                    "Could not send encrypted message as no encryption password was provided"
                )
                return

            iv = Crypto.Random.get_random_bytes(16)
            conversation_key = self.client.get_conversation_key(target, target_type)

            text_bytes = text.encode("utf-8")
            text = CryptoUtils.encrypt_aes(text_bytes, conversation_key, iv)

        files_sent = []

        if files is not None:

            if isinstance(files, str | int):
                files = [files]

            for file in files:
                if isinstance(file, str):
                    if file.isnumeric():
                        files_sent.append(int(file))
                    else:
                        file = self.client.files.upload(
                            target, file, encrypted=encrypted
                        )
                        files_sent.append(int(file.id))

                elif isinstance(file, int):
                    files_sent.append(int(file))

                else:
                    file = self.client.files.upload(target, file, encrypted=encrypted)
                    files_sent.append(int(file.id))

        if isinstance(urls, str):
            sent_urls = [urls]
        else:
            sent_urls = urls

        data = {
            "target": target_type,
            f"{target_type}_id": target,
            "text": text,
            "files": json.dumps(files_sent),
            "url": json.dumps(sent_urls),
            "encrypted": encrypted,
            "verification": "",
            "type": "text",
            "is_forwarded": False,
        }

        if encrypted:
            data["iv"] = iv.hex()
            data["text"] = text.hex()

        if markdown:
            data["metainfo"] = json.dumps({"v": 1, "style": "md"})

        data.update(kwargs)

        if location is True:

            location = self.client.account.location()

            if encrypted:
                data["latitude"] = CryptoUtils.encrypt_aes(
                    str(location["latitude"]).encode("utf-8"), conversation_key, iv=iv
                ).hex()
                data["longitude"] = CryptoUtils.encrypt_aes(
                    str(location["longitude"]).encode("utf-8"), conversation_key, iv=iv
                ).hex()
            else:
                data["latitude"] = str(location["latitude"])
                data["longitude"] = str(location["longitude"])

        elif isinstance(location, tuple | list):

            if encrypted:
                data["latitude"] = CryptoUtils.encrypt_aes(
                    str(location[0]).encode("utf-8"), conversation_key, iv=iv
                ).hex()

                data["longitude"] = CryptoUtils.encrypt_aes(
                    str(location[1]).encode("utf-8"), conversation_key, iv=iv
                ).hex()
            else:
                data["latitude"] = str(location[0])
                data["longitude"] = str(location[1])

        data = self.client._post("message/send", data=data)["message"]
        return Message(self.client, data)

    def decode(self, target: str, text: bytes, iv: bytes, key: bytes = None) -> str:
        """## Decode a encrypted message.

        #### Args:
            target (str): The types id.
            text (bytes): The encrypted text.
            iv (bytes): The iv of the text.
            key (bytes, optional): The conversation key. Defaults to None.

        #### Returns:
            str: The decrypted key.
        """
        target_type = self.client.tools.get_type(target)

        if text == "":
            return text
        else:
            try:
                if self.client._private_key is None:
                    return text
                else:
                    conversation_key = self.client.get_conversation_key(
                        target, target_type, key=key
                    )

                    text = CryptoUtils.decrypt_aes(
                        bytes.fromhex(text), conversation_key, bytes.fromhex(iv)
                    )
                    return text.decode("utf-8")
            except Exception:
                return text

    def like(self, message_id: str | int) -> dict:
        """## Likes a message.

        #### Args:
            message_id (str | int): The messages id.

        #### Returns:
            dict: The success status.
        """
        return self.client._post("message/like", data={"message_id": message_id})

    def unlike(self, message_id: str | int) -> dict:
        """## Unlikes a message.

        #### Args:
            message_id (str | int): The messages id.

        #### Returns:
            dict: The success status.
        """
        return self.client._post("message/unlike", data={"message_id": message_id})

    def delete(self, message_id: str | int) -> dict:
        """## Deletes a message.

        #### Args:
            message_id (str | int): The messages id.

        #### Returns:
            dict: The succes status.
        """
        return self.client._post("message/delete", data={"message_id": message_id})

    def infos(self, message_ids: str | int | list) -> dict:
        """## Gets the infos of messages.

        #### Args:
            message_ids (str | int | list): The message ids.

        #### Returns:
            dict: The message infos
        """
        if isinstance(message_ids, str | int):
            ids = [message_ids]
        else:
            ids = message_ids
        messages = self.client._post(
            "message/infos", data={"message_ids": json.dumps(ids)}
        )
        return [Message(self.client, message) for message in messages["messages"]]

    def get_messages(
        self, type_id: str | int, limit: int = 30, offset: int = 0
    ) -> Generator[Message, None, None]:
        """## Gets the messages of a channel or conversation.

        #### Args:
            type_id (str | int): The types id
            limit (int, optional): The responses limit. Defaults to 30.
            offset (int, optional): The responses offset. Defaults to 0.

        #### Yields:
            Generator[Message, None, None]: Message objects.
        """
        target_type = self.client.tools.get_type(type_id)

        data = {
            f"{target_type}_id": type_id,
            "source": target_type,
            "limit": limit,
            "offset": offset,
        }

        response = self.client._post("message/content", data=data)
        response = response["messages"]

        for message in response:
            if message["kind"] != "message":
                continue

            yield Message(self.client, message)

    def get_flagged(
        self, type_id: str | int, limit: int = 100, offset: int = 0
    ) -> Generator[Message, None, None]:
        """## Gets the flagged messages of a channel.

        #### Args:
            type_id (str | int): The types id.
            limit (int, optional): The responses limit. Defaults to 100.
            offset (int, optional): The responses offset. Defaults to 0.

        #### Yields:
            Generator[Message, None, None]: Message objects.
        """
        target_type = self.client.tools.get_type(type_id)

        data = {
            "type": target_type,
            "type_id": type_id,
            "offset": offset,
            "limit": limit,
        }

        response = self.client._post("message/list_flagged_messages", data=data)
        response = response["messages"]

        for message in response:
            if message["kind"] != "message":
                continue

            yield Message(self.client, message)

    def flag(self, message_id: str | int) -> dict:
        """## Flags a message.

        #### Args:
            message_id (str | int): The messages id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post("message/flag", data={"message_id": message_id})
        return response

    def unflag(self, message_id: str | int) -> dict:
        """## Unflags a message.

        #### Args:
            message_id (str | int): The messages id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post("message/unflag", data={"message_id": message_id})
        return response



# All returnable objects are stored here

from .crypto_utils import CryptoUtils

from typing import Generator


class Message:
    def __init__(self, client, data):
        self.client = client
        self.id = data["id"]

        if data["channel_id"] == 0:
            self.type = "conversation"
            self.type_id = data["conversation_id"]
        else:
            self.type = "channel"
            self.type_id = data["channel_id"]

        self.conversation_key = self.client.get_conversation_key(
            data[f"{self.type}_id"], self.type
        )

        self.content_encrypted = data["text"]
        self.encrypted = data["encrypted"]
        self.iv = data["iv"] if self.encrypted else None

        if self.encrypted:
            self.content = self.client.messages.decode(
                data[f"{self.type}_id"], self.content_encrypted, self.iv
            )
        else:
            self.content = self.content_encrypted

        self.timestamp = data["time"]
        self.channel_id = data["channel_id"]
        self.conversation_id = data["conversation_id"]

        self.files = [File(self.client, file) for file in data["files"]]

        self.flagged = data["flagged"]

        self.liked = data["liked"]
        self.likes = data["likes"]
        self.links = data["links"]

        self._decrypt_location(data["location"])

        self.author = User(self.client, data["sender"])

    def _decrypt_location(self, location):

        if location is None:
            self.longitude = None
            self.latitude = None
            return

        if location.get("encrypted") is None or location.get("iv") is None:
            self.longitude = location["longitude"]
            self.latitude = location["latitude"]
            return
        
        if location["encrypted"]:

            if self.client._private_key is None:
                print(
                    "Could not decrypt encrypted location as no encryption password was provided"
                )
                self.longitude = location["longitude"]
                self.latitude = location["latitude"]
                return

            self.longitude = CryptoUtils.decrypt_aes(
                bytes.fromhex(location["longitude"]),
                self.conversation_key,
                bytes.fromhex(self.iv),
            ).decode("utf-8")

            self.latitude = CryptoUtils.decrypt_aes(
                bytes.fromhex(location["latitude"]),
                self.conversation_key,
                bytes.fromhex(self.iv),
            ).decode("utf-8")
        else:
            self.longitude = location["longitude"]
            self.latitude = location["latitude"]

    def like(self) -> dict:
        """## Likes a message.

        #### Returns:
            dict: The success status.
        """
        return self.client.messages.like(self.id)

    def unlike(self) -> dict:
        """## Unlikes a message.

        #### Returns:
            dict: The success status.
        """
        return self.client.messages.unlike(self.id)

    def delete(self) -> dict:
        """## Deletes a message.

        #### Returns:
            dict: The succes status.
        """
        return self.client.messages.delete(self.id)

    def flag(self) -> dict:
        """## Flags a message.

        #### Returns:
            dict: The success status.
        """
        return self.client.messages.flag(self.id)

    def unflag(self) -> dict:
        """## Unflags a message.

        #### Returns:
            dict: The success status.
        """
        return self.client.messages.unflag(self.id)

    def respond(
        self,
        text: str,
        *,
        markdown: bool = True,
        files: str | int | list = None,
        urls: str | list = "",
        location: bool | tuple | list = None,
        encrypted: bool = True,
        **kwargs,
    ):
        """## Sends a message.

        #### Args:
            text (str): The text to send.
            markdown (bool): Add markdown support. Defaults to True.
            files (str | int | list, optional): Files to send. Defaults to None.
            urls (str | list, optional): Url's to append to the message. Defaults to "".
            location (bool | tuple | list, optional): The location of the message. Defaults to None.
            encrypted (bool, optional): If the message should be encrypted. Defaults to True.

        #### Info:
            :If the file is str | int you need to give a id.
            :If its a list then you need to give a path.
            :The location needs to be set to lat, lng in a tuple or None.

        #### Returns:
            Message: A message object.
        """
        return self.client.messages.send(
            target=self.type_id,
            text=text,
            markdown=markdown,
            files=files,
            urls=urls,
            location=location,
            encrypted=encrypted,
            **kwargs,
        )


class User:
    def __init__(self, client, data) -> None:
        self.client = client
        self.id = data["id"]

        try:
            self.set_attributes(data)

        except KeyError:
            try:
                user_data = self.client.users._info(self.id)
                self.set_attributes(user_data)
            except Exception as e:
                print("could not fetch a users information - most likely due to missing permissions: ", e)

        except TypeError:
            try:
                user_data = self.client.users._info(self.id)
                self.set_attributes(user_data)
            except Exception as e:
                print("could not fetch a users information - most likely due to missing permissions: ", e)

    def set_attributes(self, data):
        self.first_name = data["first_name"]
        self.last_name = data["last_name"]

        self.email = data["email"]
        self.status = data["status"]
        self.image = data["image"]

        self.language = data["language"]
        self.last_login = data["last_login"]
        self.online = data["online"]
        self.permissions = data["permissions"]

        self.public_key = data["public_key"]
        self.companies = data["roles"]


class Conversation:
    def __init__(self, client, data):
        self.client = client
        self.id = data["id"]

        self.type = "conversation"
        self.type_id = data["id"]

        self.conversation_id = data["id"]
        self.channel_id = data["id"]

        self.key_sender = data["key_sender"]
        self.conversation_key = self.client.get_conversation_key(
            data["id"], self.type, key=data["key"]
        )

        self.encrypted = data["encrypted"]
        self.favorited = data["favorite"]
        self.archived = data["archive"]

        self.last_action = data["last_action"]
        self.last_activity = data["last_activity"]

        self.muted = data["muted"]
        self.name = data["name"]

        self.unread_messages = data["unread_messages"]
        self.user_count = data["user_count"]

        self.members = [User(self.client, member) for member in data["members"]]
        self.callable = [User(self.client, member) for member in data["callable"]]

    def archive(self) -> dict:
        """## Archives a conversation.

        #### Returns:
            dict: The success status.
        """
        return self.client.conversations.archive(self.id)

    def favorite(self) -> dict:
        """## Favorites a conversation.

        #### Returns:
            dict: The success status.
        """
        return self.client.conversations.favorite(self.id)

    def unfavorite(self) -> dict:
        """## Unfavorites a conversation.

        #### Returns:
            dict: The success status.
        """
        return self.client.conversations.unfavorite(self.id)

    def disable_notifications(self, duration: int | str) -> str:
        """## Disables notifications for a conversation.

        #### Args:
            duration (int | str): how long the block should last (seconds).

        #### Returns:
            str: The end timestamp.
        """
        return self.client.conversations.disable_notifications(self.id, duration)

    def enable_notifications(self) -> dict:
        """## Enables notifications for a conversation.

        #### Returns:
            dict: The success status.
        """
        return self.client.conversations.enable_notifications(self.id)


class Company:
    def __init__(self, client, data):
        self.client = client

        if "company_id" in data:
            data = self.client._post(
                "company/details", data={"company_id": data["company_id"]}
            )["company"]

        self.id = data["id"]

        self.name = data["name"]
        self.manager = User(self.client, data["manager"])

        self.time_created = data["created"]
        self.time_joined = data["time_joined"]
        self.unread_messages = data["unread_messages"]

        self.logo_url = data["logo_url"]
        self.domain = data["domain"]

        self.max_users = data["max_users"]
        self.active_users = data["users"]["active"]
        self.created_users = data["users"]["created"]

        self.membership_expiry = data["membership_expiry"]
        self.online_payment = data["online_payment"]
        self.protected = data["protected"]

        self.provider = data["provider"]
        self.quota = data["quota"]
        self.freemium = data["freemium"]

        self.deactivated = data["deactivated"]
        self.deleted = data["deleted"]
        self.features = data["features"]

        self.permission = data["permission"]
        self.roles = data["roles"]
        self.settings = data["settings"]

    def get_settings(self) -> dict:
        """## Gets the settings of a company.

        #### Returns:
            dict: The companies settings.
        """
        return self.client.companies.get_settings(self.id)

    def email_templates(self) -> dict:
        """## Gets the email templates of the company.

        #### Returns:
            dict: Return a dict i think
        """
        return self.client.companies.email_templates(self.id)

    def get_ldaps(self) -> dict:
        """## Gets the companies ldaps [untested].

        #### Returns:
            dict: [untested]
        """
        return self.client.companies.get_ldaps(self.id)

    def delete(self) -> dict:
        """## Deletes the company [dangerous!].

        #### Returns:
            dict: The success status.
        """
        return self.client.companies.delete(self.id)

    def quit(self) -> dict:
        """## Leaves a company [dangerous!].

        #### Returns:
            dict: The success status.
        """
        return self.client.companies.quit(self.id)

    def list_features(self) -> dict:
        """## Lists company features.

        #### Returns:
            dict: Company features
        """
        return self.client.companies.list_features(self.id)

    def get_market(self) -> dict:
        """## Gets the companies market.

        #### Returns:
            dict: The market
        """
        return self.client.companies.get_market(self.id)


class Channel:
    def __init__(self, client, data):
        self.client = client
        self.id = data["id"]

        try:
            self.set_attributes(data)

        except KeyError:
            data = self.client.channels._info(self.id)
            self.set_attributes(data)

        except TypeError:
            data = self.client.channels._info(self.id)
            self.set_attributes(data)

    def set_attributes(self, data):
        self.company = Company(self.client, {"company_id": data["company"]})

        self.crypto_properties = data["crypto_properties"]
        self.encrypted = data["encrypted"]
        self.federated = data["federated"]
        self.unique_identifier = data["unique_identifier"]

        self.description = data["description"]
        self.name = data["name"]
        self.image = data["image"]
        self.group_id = data["group_id"]

        self.can_leave = data["can_leave"]
        self.inviteable = data["inviteable"]
        self.last_action = data["last_action"]

        self.ldap_name = data["ldap_name"]
        self.mx_room_alias = data["mx_room_alias"]
        self.mx_room_id = data["mx_room_id"]
        self.mx_room_server_status = data["mx_room_server_status"]

        self.num_members_without_keys = data["num_members_without_keys"]
        self.password = data["password"]
        self.pending_count = data["pending_count"]
        self.request_count = data["request_count"]

        self.show_activities = data["show_activities"]
        self.show_membership_activities = data["show_membership_activities"]
        self.type = data["type"]

        self.user_count = data["user_count"]
        self.visible = data["visible"]
        self.writable = data["writable"]

        self.is_member = data["membership"]["is_member"]
        self.joined = data["membership"]["joined"]
        self.may_manage = data["membership"]["may_manage"]
        self.muted = data["membership"]["muted"]
        self.write = data["membership"]["write"]

        self.confirmation = data["membership"]["confirmation"]
        self.invited_at = data["membership"]["invited_at"]
        self.invited_by = data["membership"]["invited_by"]
        self.invited_by_mx_user_id = data["membership"]["invited_by_mx_user_id"]

    def edit(
        self,
        *,
        description: str = "",
        channel_name: str,
        password: str = None,
        visible: bool = True,
        writable: str = "all",
        inviteable: str = "all",
        show_activities: bool = True,
        show_membership_activities: bool = True,
    ):
        """## Edits a channel.

        #### Args:
            channel_name (str): The channels name.
            description (str, optional): The channels description. Defaults to "".
            password (str, optional): The channels password. Defaults to None.
            visible (bool, optional): The channels visibility. Defaults to True.
            writable (str, optional): Sets who can write in the channel. Defaults to "all".
            inviteable (str, optional): Sets who can invite other users. Defaults to "all".
            show_activities (bool, optional): [name]. Defaults to True.
            show_membership_activities (bool, optional): [name]. Defaults to True.

        #### Returns:
            Channel: A channel object.
        """

        return self.client.channels.edit(
            self.company.id,
            self.id,
            description=description,
            channel_name=channel_name,
            password=password,
            visible=visible,
            writable=writable,
            inviteable=inviteable,
            show_activities=show_activities,
            show_membership_activities=show_membership_activities,
        )

    def quit(self) -> dict:
        """## Leaves a channel.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.quit(self.id)

    def rename(self, channel_name: str) -> dict:
        """## Renames a channel.

        #### Args:
            channel_name (str): The new channel name.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.rename(self.id, channel_name)

    def edit_description(self, description: str) -> dict:
        """## Edits the description of a channel.

        #### Args:
            description (str): The new channel description.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.edit_description(self.id, description)

    def delete(self) -> dict:
        """## Deletes a channel (without confirmation!).

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.delete(self.id)

    def change_permission(self, writable: str):
        """## Sets who can write in the channel.

        #### Args:
            writable (str): Sets who can write in the channel.

        #### Returns:
            Channel: A channel object.
        """
        return self.client.channels.change_permission(self.id, writable)

    def remove_user(self, user_id: int | str):
        """## Removes the user from the channel.

        #### Args:
            user_id (int | str): The users id.

        #### Returns:
            Channel: A channel object.
        """
        return self.client.channels.remove_user(self.id, user_id)

    def add_manager_status(self, user_id: int | str):
        """## Adds a moderation status.

        #### Args:
            user_id (int | str): The users id.

        #### Returns:
            Channel: A channel object.
        """
        return self.client.channels.add_manager_status(self.id, user_id)

    def remove_manager_status(self, user_id: int | str):
        """## Removes a moderation status.

        #### Args:
            user_id (int | str): The users id.

        #### Returns:
            Channel: A channel object.
        """
        return self.client.channels.remove_manager_status(self.id, user_id)

    def edit_password(self, password: str) -> dict:
        """## Edits the password of the channel.

        #### Args:
            password (str): The new password.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.edit_password(self.id, password)

    def invite(
        self,
        members: int | str | list | tuple,
        text: str = "",
        expiry: int | str = None,
    ) -> dict:
        """## Creates an invite for a channel.

        #### Args:
            members (int | str | list | tuple): Members to invite as a list or string.
            text (str, optional): The text invited users will become. Defaults to "".
            expiry (int | str, optional): Expiry time as a unix timestamp. Defaults to None.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.invite(self.id, members, text, expiry)

    def members(
        self, *, search: str | int = None, limit: int | str = 40, offset: int | str = 0
    ) -> Generator[User, None, None]:
        """## Lists the members if a channel as a generator.

        #### Args:
            search (str | int, optional): The search keyword that is used. Defaults to None.
            limit (int | str, optional): Limit of answer. Defaults to 40.
            offset (int | str, optional): Offset of answer. Defaults to 0.

        #### Yields:
            Generator[User, None, None]: A generator object with a User object
            (use: for member in members).
        """
        return self.client.channels.members(
            self.id, search=search, limit=limit, offset=offset
        )

    def join(self, *, password: str | int = ""):
        """## Joins a channel.

        #### Args:
            password (str | int, optional): The password. Defaults to "".

        #### Returns:
            Channel: A channel object.
        """
        return self.client.channels.join(self.id, password=password)

    def favorite(self) -> dict:
        """## Favorites a channel.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.favorite(self.id)

    def unfavorite(self) -> dict:
        """## Unfavorites a channel.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.unfavorite(self.id)

    def disable_notifications(self, duration: int | str) -> dict:
        """## Disables notifications for a channel.

        #### Args:
            duration (int | str): how long the block should last (seconds).

        #### Returns:
            dict: The end timestamp.
        """
        return self.client.channels.disable_notifications(self.id, duration)

    def enable_notifications(self) -> dict:
        """## Enables notifications for a channel.

        #### Returns:
            dict: The success status.
        """
        return self.client.channels.enable_notifications(self.id)


class File:
    def __init__(self, client, data):
        self.client = client
        self.id = data["id"]

        try:
            self.set_attributes(data)

        # happens when key is missing
        except KeyError:
            data = self.client.files._info(self.id)
            self.set_attributes(data)

        # happens when no owner is found
        except TypeError:
            data = self.client.files._info(self.id)
            self.set_attributes(data)

    def set_attributes(self, data):
        self.name = data["name"]

        self.virtual_folder = data["virtual_folder"]
        self.folder_type = data["folder_type"]
        self.type_id = data["type_id"]

        self.size = data["size"]
        self.size_byte = data["size_byte"]
        self.size_string = data["size_string"]

        self.width = data["dimensions"]["width"]
        self.height = data["dimensions"]["height"]

        self.extension = data["ext"]
        self.mimetype = data["mime"]

        self.base_64 = data["base_64"]

        self.uploaded = data["uploaded"]
        self.modified = data["modified"]

        self.permission = data["permission"]

        self.owner_id = data["owner_id"]
        self.owner = User(self.client, data["owner"])

        self.last_download = data["last_download"]
        self.times_downloaded = data["times_downloaded"]

        self.status = data["status"]
        self.deleted = data["deleted"]
        self.encrypted = data["encrypted"]

        self.iv = data["e2e_iv"]
        self.md5 = data["md5"]

    def store_preview_image(self, filepath: str):
        """## Stores a preview image for a file.

        #### Args:
            filepath (str): The images file path.

        #### Returns:
            File | dict: A file object or a status: false dict.
        """
        return self.client.files.store_preview_image(self.id, filepath)

    def download(self, directory: str = "", filename: str = None) -> str:
        """## Downloads a file to a local location.

        #### Args:
            directory (str, optional): The download dir. Defaults to main.
            filename (str, optional): The new filename. Defaults to the main name.

        #### Returns:
            str: The path of the saved file.
        """
        return self.client.files.download(self.id, directory, filename)

    def download_bytes(self) -> bytes:
        """## Downloads a file and returns its content as bytes.

        #### Returns:
            str: The path of the saved file.
        """
        return self.client.files.download_bytes(self.id)

    def delete(self) -> dict:
        """## Deletes specified files.

        #### Returns:
            dict: The success status.
        """
        return self.client.files.delete(self.id)

    def move(self, folder_id: str | int) -> dict:
        """## Moves a file into a specified folder.

        #### Args:
            folder_id (str | int): The folders id.

        #### Returns:
            dict: The success status.
        """
        return self.client.files.move(self.id, folder_id)

    def rename(self, name: str) -> dict:
        """## Renames a file.

        #### Args:
            name (str): The files new name.

        #### Returns:
            dict: The success status
        """
        return self.client.files.rename(self.id, name)

    def shares(self) -> dict:
        """## Get a files shares.

        #### Returns:
            dict: The files shares as a dict.
        """
        return self.client.files.shares(self.id)



from .models import User


class UserManager:
    def __init__(self, client):
        self.client = client

    def _info(self, user_id: str | int, withkey: bool = True) -> dict:
        """## Gets a users user info as a dict.

        #### Args:
            user_id (str | int): The users id
            withkey (bool, optional): Return key. Defaults to True.

        #### Returns:
            dict: A user as a dict.
        """
        response = self.client._post(
            "users/info", data={"user_id": user_id, "withkey": withkey}
        )
        return response["user"]

    def info(self, user_id: str | int, withkey: bool = True) -> User:
        """## Gets a users user info.

        #### Args:
            user_id (str | int): The users id
            withkey (bool, optional): Return key. Defaults to True.

        #### Returns:
            User: A user object.
        """
        response = self.client._post(
            "users/info", data={"user_id": user_id, "withkey": withkey}
        )
        return User(self.client, response["user"])

    def me(self) -> User:
        """## Gets the clients user object.

        #### Returns:
            User: A user object.
        """
        response = self.client._post("users/me", data={})
        return User(self.client, response["user"])



class Tools:
    def __init__(self, client):
        self.client = client

    def get_type(self, type_id):
        """## Returns a location type.

        #### Args:
            type_id (int | str): The conversation or channel id
        """
        if type_id == self.client.user_id:
            return "personal"

        conversation_data = {
            "conversation_id": type_id,
            "source": "conversation",
            "limit": 0,
            "offset": 0,
        }
        channel_data = {
            "channel_id": type_id,
            "source": "channel",
            "limit": 0,
            "offset": 0,
        }
        try:
            self.client._post("message/content", data=conversation_data)
            return "conversation"
        except Exception:
            try:
                self.client._post("message/content", data=channel_data)
                return "channel"
            except Exception:
                return "404"



import Crypto
import Crypto.Random
import Crypto.Util

import os
import mimetypes
import uuid
from PIL import Image
from io import BytesIO
import base64
import json

from .crypto_utils import CryptoUtils
from .models import Channel, Conversation, File


class FileManager:
    def __init__(self, client):
        self.client = client

    def quota(self) -> dict:
        """## Gets the users quota.

        #### Returns:
            dict: The users quota.
        """
        response = self.client._post(
            "file/quota", data={"type": "personal", "type_id": self.client.user_id}
        )
        return response["quota"]

    def upload(
        self,
        target: str | int,
        file_input: str | BytesIO | bytes,
        filename: str = "stashconnect_file",
        encrypted: bool = True,
        preview: bool = True,
    ) -> File:
        """## Uploads a file to a target location.

        #### Args:
            target (str | int): The upolads target id.
            filepath (str | BytesIO | bytes): The files location path or a file-like object or bytes.
            filename(str): Only needed for bytes and BytesIO. Defaults to "file".
            encrypted (bool, optional): Sets whether a file should be encrypted. Defaults to True.
            preview (bool, optional): Sets whether a preview image should be set. Defaults to True.

        #### Returns:
            File: A file object.
        """
        if isinstance(file_input, BytesIO):
            # if file_input is a file-like object
            file_content = file_input.read()
            filename = getattr(file_input, "name", filename)

        elif isinstance(file_input, bytes):
            # if the file_input is bytes
            file_content = file_input
            filename = filename

        else:
            # if the file_input is a filepath
            if filename != "stashconnect_file":
                extension = os.path.splitext(file_input)[1]
                filename += extension
            else:
                filename = os.path.basename(file_input)

            with open(file_input, "rb") as file:
                file_content = file.read()

        if encrypted:
            if self.client._private_key is None:
                print(
                    "Could not upload encrypted file as no encryption password was provided"
                )
                return

            # generate random iv and file key
            iv = Crypto.Random.get_random_bytes(16)
            file_key = Crypto.Random.get_random_bytes(32)

        # guess content type from extension
        content_type = mimetypes.guess_type(filename)[0]
        if not content_type:
            content_type = "application/octet-stream"

        max_chunk_size = 5 * 1024 * 1024  # limit chunk upload size to 5MB
        upload_identifier = str(uuid.uuid4())  # the uploads id

        try:
            if isinstance(file_input, BytesIO | bytes):
                image = Image.open(BytesIO(file_content))
            else:
                image = Image.open(file_input)

            image_width = image.width
            image_height = image.height
        except Exception:
            image_width = None
            image_height = None

        # calculate total chunks
        total_chunks = (len(file_content) + max_chunk_size - 1) // max_chunk_size
        target_type = self.client.tools.get_type(target)

        for i in range(total_chunks):
            data_chunk = file_content[i * max_chunk_size : (i + 1) * max_chunk_size]

            # encrypt the chunk
            if encrypted:
                encrypted_chunk = CryptoUtils.encrypt_aes(data_chunk, file_key, iv)
            else:
                encrypted_chunk = data_chunk

            data = {
                "resumableChunkNumber": i,
                "resumableChunkSize": max_chunk_size,
                "resumableCurrentChunkSize": len(encrypted_chunk),
                "resumableTotalSize": len(file_content),
                "resumableType": content_type,
                "resumableIdentifier": upload_identifier,
                "resumableFilename": filename,
                "resumableRelativePath": filename,
                "resumableTotalChunks": total_chunks,
                "folder": 0,
                "type": target_type,
                "type_id": target,
                "encrypted": encrypted,
                "media_width": image_width,
                "media_height": image_height,
            }

            if encrypted:
                data["iv"] = iv.hex()

            files = {
                "file": ("[object Object]", encrypted_chunk, "application/octet-stream")
            }

            # upload the current chunk
            response = self.client._post("file/upload", data=data, files=files)
            file = response["file"]

        file_id = file["id"]

        if encrypted:
            # sets a file access key for encrypted files

            iv = Crypto.Random.get_random_bytes(16)

            data = {
                "file_id": file_id,
                "target": target_type,
                "target_id": target,
                "key": CryptoUtils.encrypt_aes(
                    file_key, self.client.get_conversation_key(target, target_type), iv
                ).hex(),
                "iv": iv.hex(),
            }

            response = self.client._post("security/set_file_access_key", data=data)

        if preview:
            self.client.files.store_preview_image(file_id, file_input)

        return File(self.client, file)

    def store_preview_image(self, file_id: str | int, filepath: str) -> File | dict:
        """## Stores a preview image for a file.

        #### Args:
            file_id (str | int): The files id.
            filepath (str): The images file path.

        #### Returns:
            File | dict: A file object or a status: false dict.
        """
        try:
            with Image.open(filepath) as image:
                output_size = 100

                image = image.convert("RGB")
                min_dimension = min(image.width, image.height)
                scale_factor = output_size / min_dimension

                new_width = int(image.width * scale_factor)
                new_height = int(image.height * scale_factor)

                image = image.resize((new_width, new_height), Image.Resampling.LANCZOS)
                left, top = (new_width - output_size) / 2, (
                    new_height - output_size
                ) / 2
                right, bottom = left + output_size, top + output_size

                image = image.crop((left, top, right, bottom))
                buffered = BytesIO()
                image.save(buffered, format="JPEG")

                image_base64 = base64.b64encode(buffered.getvalue()).decode("utf-8")

            data = {
                "file_id": file_id,
                "content": str("data:image/jpeg;base64," + image_base64),
            }

            response = self.client._post("file/storePreviewImage", data=data)
            return File(self.client, response["file"])

        except Exception:
            return {"success": False}

    def download(self, id: str | int, directory: str = "", filename: str = None) -> str:
        """## Downloads a file to a local location.

        #### Args:
            id (str | int): The files id.
            directory (str, optional): The download dir. Defaults to main.
            filename (str, optional): The new filename. Defaults to the main name.

        #### Returns:
            str: The path of the saved file.
        """
        response = self.client._post(f"file/download?id={id}", data={}, return_all=True)

        if filename is None:
            file_info = self.client.files._info(id)
            file_path = os.path.join(directory, file_info["name"])
        else:
            file_info = self.client.files._info(id)
            file_path = os.path.join(directory, filename + "." + file_info["ext"])

        with open(file_path, "wb") as file:

            if file_info["encrypted"]:
                if self.client._private_key is None:
                    print(
                        "Could not download encrypted content as no encryption password was provided"
                    )
                    return

                key = CryptoUtils.decrypt_aes(
                    bytes.fromhex(file_info["keys"][0]["key"]),
                    self.client.get_conversation_key(
                        file_info["keys"][0]["chat_id"],
                        file_info["keys"][0]["type"],
                        key=file_info["keys"][0]["chat_key"],
                    ),
                    bytes.fromhex(file_info["keys"][0]["iv"]),
                )
                decrypted = CryptoUtils.decrypt_aes(
                    response.content,
                    key,
                    bytes.fromhex(file_info["e2e_iv"]),
                )
                file.write(decrypted)
            else:
                file.write(response.content)

        return file_path

    def download_bytes(self, id: str | int) -> bytes:
        """## Downloads a file and returns its content as bytes.

        #### Args:
            id (str | int): The file's id.

        #### Returns:
            str: The path of the saved file.
        """
        response = self.client._post(f"file/download?id={id}", data={}, return_all=True)
        file_info = self.client.files._info(id)

        if file_info["encrypted"]:
            if self.client._private_key is None:
                print(
                    "Could not download encrypted content as no encryption password was provided"
                )
                return

            key = CryptoUtils.decrypt_aes(
                bytes.fromhex(file_info["keys"][0]["key"]),
                self.client.get_conversation_key(
                    file_info["keys"][0]["chat_id"],
                    file_info["keys"][0]["type"],
                    key=file_info["keys"][0]["chat_key"],
                ),
                bytes.fromhex(file_info["keys"][0]["iv"]),
            )
            decrypted = CryptoUtils.decrypt_aes(
                response.content,
                key,
                bytes.fromhex(file_info["e2e_iv"]),
            )
            return decrypted
        else:
            return response.content

    def _info(self, id: str | int) -> dict:
        """## Fetches the info of a file (dict).

        #### Args:
            id (str | int): The files id.

        #### Returns:
            dict: The files dict.
        """
        response = self.client._post("file/info", data={"file_id": id})
        return response["file"]

    def info(self, id: str | int) -> File:
        """## Fetches the info of a file.

        #### Args:
            id (str | int): The files id.

        #### Returns:
            File: A file object.
        """
        response = self.client._post("file/info", data={"file_id": id})
        return File(self.client, response["file"])

    def infos(self, ids: str | int | list) -> list:
        """## Fetches mutliple files.

        #### Args:
            ids (str | int | list): The files ids.

        #### Returns:
            list: A list of files.
        """
        ids_sent = []

        if isinstance(ids, str | int):
            ids_sent = [ids]
        else:
            ids_sent = ids

        response = self.client._post(
            "file/infos", data={"file_ids": json.dumps(ids_sent)}
        )

        files = [File(self.client, file) for file in response["files"]]
        return files

    def delete(self, ids: str | int | list) -> dict:
        """## Deletes specified files.

        #### Args:
            ids (str | int | list): The file or files ids

        #### Returns:
            dict: The success status.
        """

        ids_sent = []

        if isinstance(ids, str | int):
            ids_sent = [ids]
        else:
            ids_sent = ids

        response = self.client._post(
            "file/delete", data={"file_ids": json.dumps(ids_sent)}
        )
        return response

    def move(self, id: str | int, folder_id: str | int) -> dict:
        """## Moves a file into a specified folder.

        #### Args:
            id (str | int): The files id.
            folder_id (str | int): The folders id.

        #### Returns:
            dict: The success status.
        """
        data = {"file_id": id, "parent_id": folder_id}
        response = self.client._post("file/move", data=data)
        return response

    def rename(self, id: str | int, name: str) -> dict:
        """## Renames a file.

        #### Args:
            id (str | int): The files id.
            name (str): The files new name.

        #### Returns:
            dict: The success status
        """
        data = {"file_id": id, "name": name}
        response = self.client._post("file/rename", data=data)
        return response

    def copy(
        self,
        id: str | int,
        folder_id: str | int = 0,
        type_id: str | int = None,
    ) -> File:
        """## Copies a file to a folder.

        #### Args:
            id (str | int): The files id.
            folder_id (str | int, optional): The new folders id. Defaults to main.
            type (str, optional): The destinations type. Defaults to "personal".
            type_id (str | int, optional): The destinations type id. Defaults to client.user_id.

        #### Returns:
            File: A file object.
        """
        if type_id is None:
            type_id = self.client.user_id

        target_type = self.client.tools.get_type(type_id)

        data = {
            "file_id": id,
            "folder_id": folder_id,
            "type": target_type,
            "type_id": type_id,
        }
        response = self.client._post("file/copy", data=data)
        return File(self.client, response["file"])

    def shares(self, id: str | int) -> dict:
        """## Get a files shares.

        #### Args:
            id (str | int): The files id.

        #### Returns:
            dict: The files shares as a dict.
        """
        data = {"file_id": id}
        response = self.client._post("file/shares", data=data)["shares"]

        channels = []
        for channel in response["channels"]:
            channels.append(Channel(self.client, channel))

        conversations = []
        for conversation in response["conversations"]:
            conversations.append(Conversation(self.client, conversation))

        response["channels"] = channels
        response["conversations"] = conversations

        return response

    def get(
        self,
        folder_id: str | int = 0,
        type_id: str | int = None,
        folder_only: str = "no",
        offset: int | str = 0,
        limit: int | str = 75,
        search: str = None,
        sorting: str = "created_asc",
    ) -> dict:
        """## Gets the files and folders in a dir.

        #### Args:
            folder_id (str | int, optional): The folders id. Defaults to 0 (personal).
            type_id (str | int, optional): The type id. Defaults to None.
            folder_only (str, optional): Folder only response. Defaults to "no".
            offset (int | str, optional): The response offset. Defaults to 0.
            limit (int | str, optional): The response limit. Defaults to 75.
            search (str, optional): The search prompt. Defaults to None.
            sorting (str, optional): The sorting setting. Defaults to "created_asc".

        #### Returns:
            dict: A dictonary containing folder and file objects.
        """
        if type_id is None:
            type_id = self.client.user_id

        target_type = self.client.tools.get_type(type_id)
        data = {
            "folder_id": folder_id,
            "type": target_type,
            "type_id": type_id,
            "folder_only": folder_only,
            "offset": offset,
            "limit": limit,
            "search": search,
            "sorting": sorting,
        }

        response = self.client._post("folder/get", data=data)
        return response["content"]



import requests
import json
import time
import threading
import socketio

from .messages import MessageManager
from .account import AccountManager
from .users import UserManager
from .crypto_utils import CryptoUtils
from .conversations import ConversationManager
from .companies import CompanyManager
from .channels import ChannelManager
from .files import FileManager
from .authentication import AuthManager

from .tools import Tools
from .models import Message

from . import __version__

headers = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36",
    "Accept": "application/json, text/plain, */*",
    "Accept-Language": "en",
    "Cache-Control": "no-cache",
    "Accept-Encoding": "gzip, deflate, br",
    "Connection": "keep-alive",
    "Referer": "https://app.schul.cloud/",
}


class Client:
    """## Represents a client connection to Stashcat API.

    #### Attributes:
        .email (str): The user's email used for authentication.
        .password (str): The user's password.
        .device_id (str): The device_id used to log in, defaults to "stashconnect123".
        .client_key (str): The key used in submitting requests.
        .socket_id (str): The ID for the websocket connection.
        .user_id (str): The unique ID of the connected user's account.
        .image_url (str): URL to the user's profile image.
        .first_name (str): User's first name.
        .last_name (str): User's last name.
    """

    def __init__(
        self,
        *,
        email,
        password,
        proxy=None,
        cert_path=None,
        encryption_password=None,
        device_id=None,
        app_name=None,
    ):

        self.messages = MessageManager(self)
        self.tools = Tools(self)
        self.account = AccountManager(self)
        self.users = UserManager(self)
        self.files = FileManager(self)
        self.conversations = ConversationManager(self)
        self.companies = CompanyManager(self)
        self.channels = ChannelManager(self)
        self.auth = AuthManager(self)

        self.email = email
        self.password = password
        self.encryption_password = encryption_password
        self.proxy = proxy

        self.device_id = "stashconnect" if device_id is None else device_id
        self.app_name = (
            f"stashconnect v.{__version__}" if app_name is None else app_name
        )

        self._main_url = "https://api.stashcat.com/"
        self._push_url = "https://push.stashcat.com/"

        self._headers = headers
        self._session = requests.Session()
        self._session.headers.update(self._headers)
        if proxy is not None: 
            self._session.proxies.update(proxy)
        if cert_path is not None: 
            self._session.verify = cert_path

        self._login()

        self.conversation_keys = {}
        self.events = {}
        self.loops = []

        self._private_key = None
        self._ping_target = None
        self._end_time = None
        self._latency_ws = None

        if encryption_password is not None:
            self.get_private_key(encryption_password=self.encryption_password)

    def _login(self):
        response = self.auth._login(self.email, self.password, self.app_name)

        self.client_key = response["client_key"]
        self.socket_id = response["userinfo"]["socket_id"]
        self.user_id = response["userinfo"]["id"]
        self.image_url = response["userinfo"]["image"]

        self.first_name = response["userinfo"]["first_name"]
        self.last_name = response["userinfo"]["last_name"]

        if self.encryption_password is None:
            print(
                f"Logged in as {self.first_name} {self.last_name}! "
                "No encryption password was provided so some features won't work"
            )
        else:
            print(f"Logged in as {self.first_name} {self.last_name}!")

        return response

    def _post(self, url, *, data, auth=True, return_all=False, **kwargs):

        data["device_id"] = self.device_id

        if auth is True:
            data["client_key"] = self.client_key

        response = self._session.post(f"{self._main_url}{url}", data=data, **kwargs)

        response.raise_for_status()

        if not return_all:
            response = response.json()
            status = response["status"]
            payload = response["payload"]

            if status["value"] != "OK":
                # create custom exceptions
                raise Exception(status["message"])

            return payload

        else:
            return response

    def get_private_key(self, *, encryption_password: str) -> None:

        print("Importing private key. Please wait...")
        response = self._post("security/get_private_key", data={})
        encrypted_key = json.loads(response["keys"]["private_key"])

        self._private_key = CryptoUtils.load_private_key(
            encrypted_key["private"], encryption_password
        )

    def get_conversation_key(self, target, target_type, key=None):

        if self._private_key is None:
            return None

        try:
            return self.conversation_keys[target]
        except KeyError:
            encrypted_key = key

            if encrypted_key is None:
                if target_type == "conversation":
                    response = self._post(
                        "message/conversation", data={"conversation_id": target}
                    )
                    encrypted_key = response["conversation"]["key"]
                else:
                    response = self._post(
                        "channels/info",
                        data={"channel_id": target, "without_members": True},
                    )
                    encrypted_key = response["channels"]["key"]

            decrypted_key = CryptoUtils.decrypt_key(encrypted_key, self._private_key)

            self.conversation_keys[target] = decrypted_key
            return self.conversation_keys[target]

    def event(self, name):

        def decorator(func):
            def wrapper(*args):

                if func.__name__ == "message_received":
                    func(Message(self, args[0]["message"]))

                else:
                    if len(args) == 1:
                        func(args[0])
                    else:
                        func(args)

            self.events[name] = wrapper
            return wrapper

        return decorator

    def loop(self, seconds):
        def decorator(func):
            def wrapped_func():
                def run():
                    time.sleep(2)
                    while True:
                        func()
                        time.sleep(seconds)

                return run

            self.loops.append(wrapped_func())
            return func

        return decorator

    def event_modifier(self):
        def decorator(func):
            def wrapper(*args):
                if str(args[2]) == str(self.user_id) and str(args[1]) == str(
                    self._ping_target
                ):
                    self._end_time = time.perf_counter()
                func(*args)

            return wrapper

        return decorator

    def _run(self, debug=False):

        self.sio = socketio.Client(logger=debug, engineio_logger=debug)

        @self.sio.event
        def connect():

            print("Connected to the server.")

            data = {
                "hidden_id": self.socket_id,
                "device_id": self.device_id,
                "client_key": self.client_key,
            }

            self.sio.emit("userid", data)

        @self.sio.event
        def disconnect():
            print("Disconnected from the server")
            self.sio.disconnect()

        for event_name, event_handler in self.events.items():
            if event_name == "user-started-typing":
                event_modifier = self.event_modifier()(event_handler)
                self.sio.on(event_name)(event_modifier)
            else:
                self.sio.on(event_name)(event_handler)

        self.sio.connect(self._push_url)
        self.sio.wait()

    def run(self, debug=False):
        self._run_loops()
        if len(self.events) != 0:
            self._run(debug=debug)

    def _run_loops(self):
        for loop in self.loops:
            thread = threading.Thread(target=loop)
            thread.start()

    def ws_latency(self, target):
        """Gets the websockets latency (currently broken).

        Args:
            target (int|str): A conversation to use. (Use the messages destination chat)

        Returns:
            int: The websockets latency.
            str: Error.
        """
        target_type = self.tools.get_type(target)

        start_time = time.perf_counter()
        self._end_time = None
        self._ping_target = target

        self.sio.emit(
            "started-typing", (self.device_id, self.client_key, target_type, target)
        )

        time.sleep(2)

        if self._end_time is None:
            self._latency_ws = None
            return "-"
        else:
            self._latency_ws = (round((self._end_time - start_time) * 100000)) / 100
            return self._latency_ws



import Crypto.Cipher
import Crypto.Cipher.AES
import Crypto.Cipher.PKCS1_OAEP
import Crypto.PublicKey
import Crypto.PublicKey.RSA
import Crypto.Random
import Crypto

import base64
import json

from .models import User, Channel
from typing import Generator


class ChannelManager:
    def __init__(self, client):
        self.client = client

    def create(
        self,
        channel_name: str,
        company_id: int | str,
        *,
        description: str = "",
        password: str = None,
        channel_type: str = "encrypted",
        visible: bool = True,
        writable: str = "all",
        inviteable: str = "all",
        show_activities: bool = True,
        show_membership_activities: bool = True
    ) -> Channel:
        """## Creates a channel.

        #### Args:
            channel_name (str): The channels name.
            company_id (int | str): The companies id.
            description (str, optional): The channels description. Defaults to "".
            password (str, optional): The channels password. Defaults to None.
            channel_type (str, optional): The channels type. Defaults to "encrypted".
            visible (bool, optional): The channels visibility. Defaults to True.
            writable (str, optional): Sets who can write in the channel. Defaults to "all".
            inviteable (str, optional): Sets who can invite other users. Defaults to "all".
            show_activities (bool, optional): [name]. Defaults to True.
            show_membership_activities (bool, optional): [name]. Defaults to True.

        #### Returns:
            Channel: A channel object.
        """
        conversation_key = Crypto.Random.get_random_bytes(32)
        encryptor = Crypto.Cipher.PKCS1_OAEP.new(self.client._private_key.publickey())

        encrypted_key = encryptor.encrypt(conversation_key)

        data = {
            "channel_name": channel_name,
            "company": company_id,
            "password": password,
            "password_repeat": password,
            "description": description,
            "type": channel_type,
            "visible": visible,
            "writable": writable,
            "encryption_key": base64.b64encode(encrypted_key).decode("utf-8"),
            "inviteable": inviteable,
            "show_activities": show_activities,
            "show_membership_activities": show_membership_activities,
        }

        response = self.client._post("channels/create", data=data)
        return Channel(self.client, response["channel"])

    def edit(
        self,
        company_id: int | str,
        channel_id: int | str,
        *,
        description: str = "",
        channel_name: str,
        password: str = None,
        visible: bool = True,
        writable: str = "all",
        inviteable: str = "all",
        show_activities: bool = True,
        show_membership_activities: bool = True
    ) -> Channel:
        """## Edits a channel.

        #### Args:
            company_id (int | str): The companies id.
            channel_id (int | str): The channels id.
            channel_name (str): The channels name.
            description (str, optional): The channels description. Defaults to "".
            password (str, optional): The channels password. Defaults to None.
            visible (bool, optional): The channels visibility. Defaults to True.
            writable (str, optional): Sets who can write in the channel. Defaults to "all".
            inviteable (str, optional): Sets who can invite other users. Defaults to "all".
            show_activities (bool, optional): [name]. Defaults to True.
            show_membership_activities (bool, optional): [name]. Defaults to True.

        #### Returns:
            Channel: A channel object.
        """

        data = {
            "channel_id": channel_id,
            "company_id": company_id,
            "channel_name": channel_name,
            "description": description,
            "writable": writable,
            "visible": visible,
            "inviteable": inviteable,
            "password": password,
            "password_repeat": password,
            "show_activities": show_activities,
            "show_membership_activities": show_membership_activities,
        }

        response = self.client._post("channels/edit", data=data)
        return Channel(self.client, response["channel"])

    def quit(self, channel_id: int | str) -> dict:
        """## Leaves a channel.

        #### Args:
            channel_id (int | str): The channels id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post("channels/quit", data={"channel_id": channel_id})
        return response

    def rename(self, channel_id: int | str, channel_name: str) -> dict:
        """## Renames a channel.

        #### Args:
            channel_id (int | str): The channels id.
            channel_name (str): The new channel name.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "channels/rename",
            data={"channel_id": channel_id, "channel_name": channel_name},
        )
        return response

    def edit_description(self, channel_id: int | str, description: str) -> dict:
        """## Edits the description of a channel.

        #### Args:
            channel_id (int | str): The channels id.
            description (str): The new channel description.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "channels/editDescription",
            data={"channel_id": channel_id, "description": description},
        )
        return response

    def delete(self, channel_id: int | str) -> dict:
        """## Deletes a channel (without confirmation!).

        #### Args:
            channel_id (int | str): The channels id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post("channels/delete", data={"channel_id": channel_id})
        return response

    def change_permission(self, channel_id: int | str, writable: str) -> Channel:
        """## Sets who can write in the channel.

        #### Args:
            channel_id (int | str): The channels id.
            writable (str): Sets who can write in the channel.

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/changePermissions",
            data={"channel_id": channel_id, "writable": writable},
        )
        return Channel(self.client, response["channel"])

    def remove_user(self, channel_id: int | str, user_id: int | str) -> Channel:
        """## Removes the user from the channel.

        #### Args:
            channel_id (int | str): The channels id.
            user_id (int | str): The users id.

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/removeUser", data={"channel_id": channel_id, "user_id": user_id}
        )
        return Channel(self.client, response["channel"])

    def add_manager_status(self, channel_id: int | str, user_id: int | str) -> Channel:
        """## Adds a moderation status.

        #### Args:
            channel_id (int | str): The channels id.
            user_id (int | str): The users id.

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/addModeratorStatus",
            data={"channel_id": channel_id, "user_id": user_id},
        )
        return Channel(self.client, response["channel"])

    def remove_manager_status(
        self, channel_id: int | str, user_id: int | str
    ) -> Channel:
        """## Removes a moderation status.

        #### Args:
            channel_id (int | str): The channels id.
            user_id (int | str): The users id.

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/removeModeratorStatus",
            data={"channel_id": channel_id, "user_id": user_id},
        )
        return Channel(self.client, response["channel"])

    def edit_password(self, channel_id: int | str, password: str) -> dict:
        """## Edits the password of the channel.

        #### Args:
            channel_id (int | str): The channels id.
            password (str): The new password.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "channels/editPassword",
            data={"password": password, "channel_id": channel_id},
        )
        return response

    def _info(self, channel_id: int | str, without_members: bool = True) -> dict:
        """## Gets the info of a channel (dict).

        #### Args:
            channel_id (int | str): The channels id.
            without_members (bool, optional): Returns the members. Defaults to True.

        #### Returns:
            dict: The channel info as a dict.
        """
        response = self.client._post(
            "channels/info",
            data={"channel_id": channel_id, "without_members": without_members},
        )
        return response["channels"]

    def info(self, channel_id: int | str, without_members: bool = True) -> Channel:
        """## Gets the info of a channel.

        #### Args:
            channel_id (int | str): The channels id.
            without_members (bool, optional): Returns the members. Defaults to True.

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/info",
            data={"channel_id": channel_id, "without_members": without_members},
        )
        return Channel(self.client, response["channels"])

    def invite(
        self,
        channel_id: int | str,
        members: int | str | list | tuple,
        text: str = "",
        expiry: int | str = None,
    ) -> dict:
        """## Creates an invite for a channel.

        #### Args:
            channel_id (int | str): The id of the channel.
            members (int | str | list | tuple): Members to invite as a list or string.
            text (str, optional): The text invited users will become. Defaults to "".
            expiry (int | str, optional): Expiry time as a unix timestamp. Defaults to None.

        #### Returns:
            dict: The success status.
        """
        # fetch the channels key
        conversation_key = self.client.get_conversation_key(channel_id, "channel")
        users = []

        if isinstance(members, str | int):
            members = [members]

        for member in members:
            user = self.client.users._info(member)

            publickey = Crypto.PublicKey.RSA.import_key(user["public_key"])
            encryptor = Crypto.Cipher.PKCS1_OAEP.new(publickey)
            encrypted_key = encryptor.encrypt(conversation_key)

            users.append(
                {
                    "id": int(user["id"]),
                    "key": base64.b64encode(encrypted_key).decode("utf-8"),
                    "expiry": expiry,
                    "userVerified": True,
                }
            )

        response = self.client._post(
            "channels/createInvite",
            data={
                "channel_id": int(channel_id),
                "users": json.dumps(users),
                "text": text,
            },
        )

        return response

    def members(
        self,
        channel_id: int | str,
        *,
        search: str | int = None,
        limit: int | str = 40,
        offset: int | str = 0
    ) -> Generator[User, None, None]:
        """## Lists the members if a channel as a generator.

        #### Args:
            channel_id (int | str): The channels id.
            search (str | int, optional): The search keyword that is used. Defaults to None.
            limit (int | str, optional): Limit of answer. Defaults to 40.
            offset (int | str, optional): Offset of answer. Defaults to 0.

        #### Yields:
            Generator[User, None, None]: A generator object with a User object
            (use: for member in members).
        """
        data = {
            "channel_id": channel_id,
            "limit": limit,
            "offset": offset,
            "filter": "members",
            "sorting": ["first_name_asc", "last_name_asc"],
            "search": search,
        }

        response = self.client._post("channels/members", data=data)

        for member in response["members"]:
            yield User(self.client, member)

    def join(self, channel_id: int | str, *, password: str | int = "") -> Channel:
        """## Joins a channel.

        #### Args:
            channel_id (int | str): The channels id.
            password (str | int, optional): The password. Defaults to "".

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/join", data={"channel_id": channel_id, "password": password}
        )
        return Channel(self.client, response["channel"])

    def recommendations(self, company_id: int | str) -> Channel:
        """## Gets custom channel recommendations.

        #### Args:
            company_id (int | str): The companies id.

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/recommendations", data={"company": company_id}
        )
        return Channel(self.client, response["channels"])

    def visible(
        self,
        company_id: int | str,
        *,
        limit: int | str = 30,
        offset: int | str = 0,
        search: str | int = ""
    ) -> Channel:
        """## Gets all visible channels.

        #### Args:
            company_id (int | str): The companies id.
            limit (int | str, optional): The returned limit. Defaults to 30.
            offset (int | str, optional): The returned offset. Defaults to 0.
            search (str | int, optional): The search keyword. Defaults to "".

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/visible",
            data={
                "company": company_id,
                "limit": limit,
                "offset": offset,
                "search": search,
            },
        )
        return Channel(self.client, response["channels"])

    def joined(self, company_id: int | str) -> Channel:
        """## Gets all joined channels.

        #### Args:
            company_id (int | str): The companies id.

        #### Returns:
            Channel: A channel object.
        """
        response = self.client._post(
            "channels/subscripted", data={"company": company_id}
        )
        return Channel(self.client, response["channels"])

    def accept_invite(self, invite_id: int | str) -> dict:
        """## Accepts an invite.

        #### Args:
            invite_id (int | str): The id of the invite.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "channels/acceptInvite", data={"invite_id": invite_id}
        )
        return response

    def decline_invite(self, invite_id: int | str) -> dict:
        """## Declines an invite.

        #### Args:
            invite_id (int | str): The id of the invite.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "channels/declineInvite", data={"invite_id": invite_id}
        )
        return response

    def favorite(self, channel_id: int | str) -> dict:
        """## Favorites a channel.

        #### Args:
            channel_id (int | str): The channels id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "message/set_favorite",
            data={"channel_id": channel_id, "favorite": True},
        )
        return response

    def unfavorite(self, channel_id: int | str) -> dict:
        """## Unfavorites a channel.

        #### Args:
            channel_id (int | str): The channels id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "message/set_favorite",
            data={"channel_id": channel_id, "favorite": False},
        )
        return response

    def disable_notifications(self, channel_id: int | str, duration: int | str) -> dict:
        """## Disables notifications for a channel.

        #### Args:
            channel_id (int | str): The channels id.
            duration (int | str): how long the block should last (seconds).

        #### Returns:
            dict: The end timestamp.
        """
        return self.client._post(
            "push/disable_notifications",
            data={
                "type": "channel",
                "content_id": channel_id,
                "duration": duration,
            },
        )

    def enable_notifications(self, channel_id: int | str) -> dict:
        """## Enables notifications for a channel.

        #### Args:
            channel_id (int | str): The channels id.

        #### Returns:
            dict: The success status.
        """
        return self.client._post(
            "push/enable_notifications",
            data={"type": "channel", "content_id": channel_id},
        )



from setuptools import setup, find_packages
from stashconnect import __version__

VERSION = __version__
DESCRIPTION = "An API wrapper for stashcat and schul.cloud."

setup(
    name="stashconnect",
    version=VERSION,
    author="BuStudios",
    author_email="support@bustudios.org",
    description=DESCRIPTION,
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/BuStudios/StashConnect",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Development Status :: 4 - Beta",
    ],
    keywords=[
        "stashconnect",
        "stashcat api",
        "stashcat python",
        "schul.cloud api",
        "schul.cloud python",
        "stashcat bot",
        "schul.cloud bot",
    ],
    install_requires=[
        "requests",
        "pycryptodome",
        "python-socketio",
        "Pillow",
        "websocket-client",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/BuStudios/StashConnect/issues",
        "Documentation": "https://github.com/BuStudios/StashConnect/wiki",
        "Source Code": "https://github.com/BuStudios/StashConnect",
    },
    python_requires=">=3.10",
)



import Crypto
import Crypto.Cipher
import Crypto.Cipher.PKCS1_OAEP
import Crypto.Hash
import Crypto.Hash.SHA256
import Crypto.PublicKey
import Crypto.PublicKey.RSA
import Crypto.Random
import Crypto.SelfTest
import Crypto.Signature
import Crypto.Signature.pkcs1_15
import Crypto.Util

import base64
import json

from .models import Conversation


class ConversationManager:
    def __init__(self, client):
        self.client = client

    def archive(self, conversation_id: str | int) -> dict:
        """## Archives a conversation.

        #### Args:
            conversation_id (str | int): The conversations id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "message/archiveConversation", data={"conversation_id": conversation_id}
        )
        return response

    def favorite(self, conversation_id: str | int) -> dict:
        """## Favorites a conversation.

        #### Args:
            conversation_id (str | int): The conversations id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "message/set_favorite",
            data={"conversation_id": conversation_id, "favorite": True},
        )
        return response

    def unfavorite(self, conversation_id: str | int) -> dict:
        """## Unfavorites a conversation.

        #### Args:
            conversation_id (str | int): The conversations id.

        #### Returns:
            dict: The success status.
        """
        response = self.client._post(
            "message/set_favorite",
            data={"conversation_id": conversation_id, "favorite": False},
        )
        return response

    def disable_notifications(
        self, conversation_id: int | str, duration: int | str
    ) -> str:
        """## Disables notifications for a conversation.

        #### Args:
            conversation_id (int | str): The conversations id.
            duration (int | str): how long the block should last (seconds).

        #### Returns:
            str: The end timestamp.
        """
        return self.client._post(
            "push/disable_notifications",
            data={
                "type": "conversation",
                "content_id": conversation_id,
                "duration": duration,
            },
        )

    def enable_notifications(self, conversation_id: int | str) -> dict:
        """## Enables notifications for a conversation.

        #### Args:
            conversation_id (int | str): The conversations id.

        #### Returns:
            dict: The success status.
        """
        return self.client._post(
            "push/enable_notifications",
            data={"type": "conversation", "content_id": conversation_id},
        )

    def create(self, members: str | int | list) -> Conversation:
        """## Creates a conversation with users.

        #### Args:
            members (str | int | list): The members of the conversation.

        #### Returns:
            Conversation: A conversation object.
        """
        conversation_key = Crypto.Random.get_random_bytes(32)
        users = []

        # encrypt conversation key using private key
        encryptor = Crypto.Cipher.PKCS1_OAEP.new(self.client._private_key.publickey())
        encrypted_key = encryptor.encrypt(conversation_key)

        # i dont know where the private signing key is located
        # if you know where it is please tell me :)

        # hash = Crypto.Hash.SHA256.new(encrypted_key)
        # signature = Crypto.Signature.pkcs1_15.new(self.client._private_key).sign(hash)
        # encoded_signature = base64.b64encode(signature).decode("utf-8")

        users.append(
            {
                "id": int(self.client.user_id),
                "key": base64.b64encode(encrypted_key).decode("utf-8"),
                # "signature": encoded_signature,
                # "userVerified": True,
            }
        )

        if isinstance(members, str | int):
            members = [members]

        # encrypt conversation key using public key for all members
        for member in members:
            user = self.client.users._info(member)

            publickey = Crypto.PublicKey.RSA.import_key(user["public_key"])

            encryptor = Crypto.Cipher.PKCS1_OAEP.new(publickey)
            encrypted_key = encryptor.encrypt(conversation_key)

            # hash = Crypto.Hash.SHA256.new(encrypted_key)
            # signature = Crypto.Signature.pkcs1_15.new(self.client._private_key).sign(hash)
            # encoded_signature = base64.b64encode(signature).decode("utf-8")

            users.append(
                {
                    "id": int(user["id"]),
                    "key": base64.b64encode(encrypted_key).decode("utf-8"),
                    # "signature": encoded_signature,
                    # "expiry": int(round(time.time())),
                    # "userVerified": True,
                }
            )

        response = self.client._post(
            "message/createEncryptedConversation",
            data={"members": json.dumps(users), "unique_identifier": conversation_key},
        )

        return Conversation(self.client, response["conversation"])

    def info(self, conversation_id: str | int) -> Conversation:
        """## Fetches the info of a conversation.

        #### Args:
            conversation_id (str | int): The conversations info.

        #### Returns:
            Conversation: A conversation object.
        """
        response = self.client._post(
            "message/conversation", data={"conversation_id": conversation_id}
        )
        return Conversation(self.client, response["conversation"])
